#!/usr/bin/python -OO
# -*- coding: utf-8 -*-
"""
slapd-sock dummy no-op listener demon queried by OpenLDAP's slapd-sock
"""

import os, logging
import slapdsock

__version__ = slapdsock.__version__
__author__ = u'Michael Ströder <michael@stroeder.com>'

#-----------------------------------------------------------------------
# Configuration constants
#-----------------------------------------------------------------------

# UIDs and peer GIDS of peers which are granted access
# (list of int/strings)
ALLOWED_UIDS = [0, 'system_slapd', os.getuid()]
ALLOWED_GIDS = [0]

# String with octal representation of socket permissions
SOCKET_PERMISSIONS = '0666'

# Timeout in seconds for the server (Unix domain) socket
SOCKET_TIMEOUT = 10

# Logging formats
SYS_LOG_FORMAT = '%(levelname)s %(message)s'
CONSOLE_LOG_FORMAT = '%(asctime)s %(levelname)s %(message)s'

# Base number for floating average value of response delay
AVERAGE_COUNT = 100

# Default log level to use
LOG_LEVEL = logging.INFO

#-----------------------------------------------------------------------
# Imports
#-----------------------------------------------------------------------

# from Python's standard lib
import sys

# local modules
from slapdsock.loghelper import combined_logger
from slapdsock.service import SlapdSockServer
from slapdsock.handler import NoopHandler

#-----------------------------------------------------------------------
# Main
#-----------------------------------------------------------------------

script_name = sys.argv[0]

log_level = LOG_LEVEL
console_log_format = None
if __debug__ and os.environ.get('DEBUG', 'no') == 'yes':
    log_level = logging.DEBUG
    console_log_format = CONSOLE_LOG_FORMAT

my_logger = combined_logger(
    script_name,
    log_level,
    sys_log_format=SYS_LOG_FORMAT,
    console_log_format=console_log_format,
)

my_logger.info(
    'Starting %s %s (log level %d)',
    script_name,
    __version__,
    my_logger.level
)

try:
    socket_path = sys.argv[1]
except IndexError:
    my_logger.error('Not enough arguments => abort')
    sys.exit(1)

try:
    slapd_sock_listener = SlapdSockServer(
        socket_path,
        NoopHandler,
        my_logger,
        AVERAGE_COUNT,
        SOCKET_TIMEOUT, SOCKET_PERMISSIONS,
        ALLOWED_UIDS, ALLOWED_GIDS,
    )
    try:
        slapd_sock_listener.serve_forever()
    except KeyboardInterrupt:
        my_logger.warn('Received interrupt signal => shutdown')
finally:
    my_logger.debug('Remove socket path %s', repr(socket_path))
    try:
        os.remove(socket_path)
    except OSError:
        pass
